<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML;

use InvalidArgumentException;
use Infomaker\Everyware\NewsML\Interfaces\NewsItemPresentation;

class IdfBodyParser
{

    /**
     * Contains a sortable map of all items
     *
     * @var array
     */
    private $itemMap = [];

    /**
     * Contains a list of all valid body items
     *
     * @var array
     */
    private $body = [];

    public function __construct(string $newsmlBody)
    {
        if (empty($newsmlBody)) {
            throw new InvalidArgumentException('Will not parse empty newsML');
        }

        $transformer = NewsMLTransformerManager::createTransformer();
        $bodyElements = $transformer->transform($newsmlBody);

        foreach ($bodyElements as $index => $bodyElement) {
            if ($bodyElement instanceof Item) {
                $this->addItem($bodyElement);
            }
        }
    }

    private function addItem(Item $item): void
    {
        $this->itemMap[] = $item->getType();
        $this->body[] = $item;
    }

    public function allOfType(string $type): array
    {
        $items = [];

        foreach ($this->allPosOfType($type) as $key) {
            $items[] = $this->body[$key];
        }

        return $items;
    }

    public function allOfTypes(array $types): array
    {
        /* the inner empty array covers cases when no loops were made */
        $items = [[]];

        foreach ($types as $type) {
            $items[] = $this->allOfType($type);
        }

        return array_merge(...$items);
    }

    public function allPosOfType(string $type): array
    {
        return array_keys($this->itemMap, $type);
    }

    public function firstOfType(string $type)
    {
        $allOfType = $this->allOfType($type);

        return array_shift($allOfType);
    }

    public function firstType()
    {
        return collect($this->itemMap)->first();
    }

    public function getBody(): array
    {
        return $this->body;
    }

    public function getItemCount(): int
    {
        return count($this->body);
    }

    public function getTypes(): array
    {
        return array_values(array_unique($this->itemMap));
    }

    public function generateContent(NewsItemPresentation $presentation): array
    {
        $convertedBody = [];

        foreach ($this->getBody() as $item) {
            if ( ! $item instanceof Item) {
                continue;
            }
            $convertedBody[] = $presentation->generateItem($item->getType(), $item);
        }

        return $convertedBody;
    }

    public function hasType(string $type): bool
    {
        return \in_array($type, $this->itemMap, true);
    }

    public function removeType(string $type): void
    {
        foreach ($this->allPosOfType($type) as $key) {
            unset($this->itemMap[$key], $this->body[$key]);
        }
    }
}
