<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\LinkParsers\AuthorLinkParser;
use Infomaker\Everyware\NewsML\LinkParsers\CropLinkParser;
use Infomaker\Everyware\NewsML\LinkParsers\FocalPointLinkParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

/**
 * ImageLinkParser
 * @package Infomaker\Everyware\NewsML\Parsers
 */
class ImageLinkParser implements ItemParser, TypeParser
{
    public const LINK_TYPE = 'x-im/image';

    public const DEFAULT_DATA = [
        'width' => '0',
        'height' => '0',
        'text' => '',
        'alttext' => ''
    ];

    /**
     * @var LinkParser
     */
    protected $linkParser;

    public function __construct()
    {
        $this->linkParser = new LinkParser([
            AuthorLinkParser::LINK_TYPE => new AuthorLinkParser(),
            CropLinkParser::LINK_TYPE => new CropLinkParser(),
            FocalPointLinkParser::LINK_TYPE => new FocalPointLinkParser()
        ]);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new ImageItem($element);
        $item->fill(array_replace($this->getImageData($item), $this->getImageLinks($element)));

        return [$item];
    }

    public function getImageLinks(SimpleXMLIterator $element): array
    {
        $links = [];
        foreach ($this->linkParser->parseLinks($element) as $link) {
            if ($link instanceof Item) {

                $attributes = $link->getItemAttributes();

                switch ($attributes['type']) {
                    case CropLinkParser::LINK_TYPE:
                        $title = $attributes['title'] ?? '';
                        if ($title !== '') {
                            $links['crop'][$title] = $link;
                        } else {
                            $links[$attributes['rel'] ?? 'link'][] = $link;
                        }
                        break;
                    case AuthorLinkParser::LINK_TYPE:
                        $links['authors'][] = $link;
                        break;
                    default:
                        $links[$attributes['rel'] ?? 'link'][] = $link;
                }
            }
        }

        return $links;
    }

    protected function getImageData(Item $item): array
    {
        $allowedTags = [
            'mark',
            'em',
            'strong',
            'a',
        ];

        $data = $item->getFirstChild('data');

        if ( ! $data instanceof Item) {
            return static::DEFAULT_DATA;
        }

        $optionalData = array_filter([
            'alignment' => $data->get('alignment'),
            'credit' => $data->get('credit'),
            'instructions' => $data->get('imageInstructions'),
            'flags' => $this->getFlags($data),
            'width' => $data->get('width'),
            'height' => $data->get('height'),
            'text' => $data->getFirstChildAsXml('text', $allowedTags),
            'alttext' => $data->get('alttext')
        ]);

        return array_replace(static::DEFAULT_DATA, $optionalData);
    }

    public function getFlags(Item $data): array
    {
        $flagsItem = $data->getFirstChild('flags');

        if ( ! $flagsItem instanceof Item) {
            return [];
        }

        return array_map(static function (Item $flag) {
            return ['name' => $flag->text()];
        }, $flagsItem->getChild('flag'));
    }

    public function typeToParse(): string
    {
        return static::LINK_TYPE;
    }
}
