<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\LinkParsers\ArticleLinkParser;
use SimpleXMLIterator;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Objects\TeaserItem;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;

/**
 * TeaserParser
 * @package Infomaker\Everyware\NewsML\Parsers
 */
class TeaserParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/teaser';

    public const DEFAULT_DATA = [
        'title' => '',
        'text' => [],
        'subject' => ''
    ];

    /**
     * @var LinkParser
     */
    protected $linkParser;

    /**
     * @var ItemParser[]
     */
    protected $textParsers;

    public function __construct(ElementParser $parser = null, LinkParser $linkParser = null)
    {
        $this->textParsers = [
            ElementParser::TAG => $parser ?? new ElementParser()
        ];

        $this->linkParser = $linkParser ?? new LinkParser([
                ImageLinkParser::LINK_TYPE => new ImageLinkParser(),
                ArticleLinkParser::LINK_TYPE => new ArticleLinkParser()
            ]);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new TeaserItem($element);
        $item->fill(
            array_replace(
                $this->getData($item), [
                'links' => $this->linkParser->parseLinks($element)
            ])
        );

        return [$item];
    }

    protected function getData(TeaserItem $teaserItem): array
    {
        $allowedTags = [
            'mark',
            'em',
            'strong',
        ];

        $data = $teaserItem->getFirstChild('data');

        if ( ! $data instanceof Item) {
            return static::DEFAULT_DATA;
        }

        return array_filter([
            'headline' => $data->getFirstChildAsXml('title', $allowedTags),
            'text' => $this->parseText($data, $allowedTags),
            'subject' => $data->getFirstChildAsXml('subject', $allowedTags),
            'alttitle' => $data->get('customText', '')
        ]);
    }

    protected function parseText(Item $dataItem, $allowedTags): string
    {
        $textItem = $dataItem->getFirstChild('text');

        if ( ! $textItem instanceof Item) {
            return '';
        }

        if ($textItem->get('format', '') !== 'idf') {
            return $textItem->getFilteredContent($allowedTags);
        }

        $textItems = [[]];
        $textIterator = simplexml_load_string($textItem->getContentAsXML(), SimpleXMLIterator::class);
        foreach ($textIterator as $tag => $el) {
            $parser = $this->textParsers[$tag] ?? null;
            $textItems[] = $parser instanceof ItemParser ? $parser->parse($el) : [new Item($el)];
        }

        $paragraphs = array_merge(...$textItems);

        return implode(' ', array_map(static function (Item $p) use ($allowedTags) {
            return $p->getFilteredContent($allowedTags);
        }, $paragraphs));
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
