<?php

namespace Spec\Infomaker\Everyware\NewsML;

use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Tightenco\Collect\Contracts\Support\Arrayable;

class ItemSpec extends ObjectBehavior
{

    public function let(): void
    {
        $this->initWithXML();
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(Item::class);
        $this->shouldImplement(Arrayable::class);
    }

    public function it_should_filter_inline_content(): void
    {
        $text = '<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>';

        $this->filterInlineContent($text, [])
            ->shouldReturn('<p>Test paragraph.</p><!-- Comment --> <a href="#fragment">Other text</a>');
        $this->filterInlineContent($text, ['a'])
            ->shouldReturn('Test paragraph. <a href="#fragment">Other text</a>');
    }

    public function it_should_get_item_attribute(): void
    {
        $this->initWithXML('<object stuff="it works!" />');
        $this->getItemAttribute('stuff')
            ->shouldReturn('it works!');
    }

    public function it_should_return_a_default_value_if_item_attribute_is_not_found(): void
    {
        $this->getItemAttribute('notFound', 'defaultValue')
            ->shouldReturn('defaultValue');
    }

    public function it_should_set_attributes_as_properties(): void
    {
        $this->initWithXML('<object stuff="it works!" />');
        $this->get('stuff')->shouldReturn('it works!');
        $this->stuff->shouldReturn('it works!');
    }

    public function it_should_return_a_child(): void
    {
        $xml = <<<XML
<object id="MjQ0LDI4LDIxNSwxNzk" title="NEW Fancy teaser with fancy article with fancy stuff" type="x-im/teaser">
    <links>
        <link name="one" game="lonely">
            <data />
        </link>
    </links>
</object>
XML;

        $this->initWithXML($xml);
        $links = $this->getFirstChild('links');

        $links->shouldBeAnInstanceOf(Item::class);

        $link = $links->getFirstChild('link');

        $link->shouldBeAnInstanceOf(Item::class);
        $link->get('name')->shouldReturn('one');
        $link->get('game')->shouldReturn('lonely');
    }

    public function it_should_get_type(): void
    {
        $this->initWithXML('<xml type="x-im/content-part" />');
        $this->getType()->shouldReturn('x-im/content-part');
    }

    public function it_should_remove_all_line_breaks(): void
    {
        $xml = <<<XML
<movies>
    <movie>abcd</movie>
    <movie>efgh</movie>
    <movie>hijk</movie>
</movies>
XML;
        $this->removeLinebreak($xml)
            ->shouldReturn('<movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }

    public function it_should_keep_spaces_during_removal_of_line_breaks(): void
    {
        $xml = <<<XML
<movies>
    <movie>abcd </movie> <movie>efgh</movie>
    <movie>hijk</movie>
</movies>
XML;
        $this->removeLinebreak($xml)
            ->shouldReturn('<movies><movie>abcd </movie> <movie>efgh</movie><movie>hijk</movie></movies>');
    }

    public function it_should_get_content_as_xml(): void
    {
        $xml = <<<XML
<movies>
    <movie>abcd</movie>
    <movie>efgh</movie>
    <movie>hijk</movie>
</movies>
XML;
        $this->initWithXML($xml);
        $this->getContentAsXML()
            ->shouldReturn('<?xml version="1.0"?><movies><movie>abcd</movie><movie>efgh</movie><movie>hijk</movie></movies>');
    }

    public function it_should_return_the_first_child(): void
    {
        $xml = <<<XML
<link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
    <data>
        <width>300</width>
        <height>169</height>
        <alttext/>
    </data>
    <data>
        <width>3000</width>
        <height>1690</height>
        <alttext/>
    </data>
</link>
XML;

        $this->initWithXML($xml);
        $data = $this->getFirstChild('data');

        $data->shouldBeAnInstanceOf(Item::class);

        $data->toArray()->shouldReturn([
            'width' => '300',
            'height' => '169',
        ]);
    }

    public function it_should_return_all_children_with_values(): void
    {
        $xml = <<<XML
<link rel="self" type="x-im/image" uri="im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg" uuid="d83dad15-9476-54c1-a57d-495757664dae">
    <data>
        <width>300</width>
        <height>169</height>
        <alttext/>
    </data>
    <data>
        <width>3000</width>
        <height>1690</height>
        <alttext/>
    </data>
</link>
XML;

        $this->initWithXML($xml);
        $this->getChild('data')
            ->shouldBeValidItemCollection([
                [
                    'width' => '300',
                    'height' => '169',
                ],
                [
                    'width' => '3000',
                    'height' => '1690',
                ],
            ]);
    }

    public function it_should_get_unfiltered_content(): void
    {
        $xml = <<<XML
<text><strong>This text</strong> <em>will be filtered</em> from <a href="#">unwanted tags</a></text>
XML;
        $this->initWithXML($xml);
        $this->getFilteredContent()
            ->shouldReturn('<?xml version="1.0"?><text><strong>This text</strong> <em>will be filtered</em> from <a href="#">unwanted tags</a></text>');
    }

    public function it_should_remove_all_but_allowed_tags_from_filtered_content(): void
    {
        $xml = <<<XML
<text><strong>This text</strong> <em>will be filtered</em> from <a href="#">unwanted tags</a></text>
XML;
        $this->initWithXML($xml);
        $this->getFilteredContent(['strong', 'em'])
            ->shouldReturn('<strong>This text</strong> <em>will be filtered</em> from unwanted tags');
    }

    public function it_should_remove_forbidden_tags_from_filtered_content(): void
    {
        $xml = <<<XML
<text><strong>This text</strong> <em>will be filtered</em> from <a href="#">unwanted tags</a></text>
XML;
        $this->initWithXML($xml);
        $this->getFilteredContent(['strong', 'em'], ['a'])
            ->shouldReturn('<strong>This text</strong> <em>will be filtered</em> from');
    }

    public function it_should_return_first_child_as_xml(): void
    {
        $xml = <<<XML
<object>
    <text>First text</text>
    <text>Second text</text>
</object>
XML;

        $this->initWithXML($xml);
        $this->getFirstChildAsXml('text')->shouldReturn('<text>First text</text>');
    }

    public function it_should_return_first_child_as_xml_with_allowed_tags(): void
    {
        $xml = <<<XML
<object>
    <text><em>hey</em><strong>there</strong><mark>general</mark><a href="#">kenobi</a></text>
</object>
XML;

        $this->initWithXML($xml);
        $this->getFirstChildAsXml('text', ['mark', 'em', 'strong', 'a'])
            ->shouldEqual('<em>hey</em><strong>there</strong><mark>general</mark><a href="#">kenobi</a>');
    }

    public function it_should_return_first_child_as_xml_with_default_value(): void
    {
        $this->initWithXML();
        $this->getFirstChildAsXml('text')->shouldreturn('');
    }

    private function initWithXML(string $xml = '<object />'): void
    {
        $this->beConstructedWith(simplexml_load_string($xml, 'SimpleXMLIterator'));
    }
}
