<?php

namespace Spec\Infomaker\Everyware\NewsML\Objects;

use Infomaker\Everyware\NewsML\Objects\ImageItem;
use Infomaker\Everyware\Support\Format;
use PhpSpec\ObjectBehavior;

class ImageItemSpec extends ObjectBehavior
{
    private $linkUri = 'im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.jpg';

    public function let(): void
    {
        $link = <<<XML
<link rel="self" type="x-im/image" uri="$this->linkUri" uuid="d83dad15-9476-54c1-a57d-495757664dae"/>
XML;

        $this->beConstructedWith(simplexml_load_string($link, 'SimpleXMLIterator'));
    }

    public function it_is_initializable(): void
    {
        $this->shouldHaveType(ImageItem::class);
    }

    public function it_should_retrieve_the_correct_crop_data(): void
    {
        $cropData = [
            '16:9' => [
                'x' => Format::formatNumberToImengineFloat(0),
                'y' => Format::formatNumberToImengineFloat(0),
                'width' => Format::formatNumberToImengineFloat(1),
                'height' => Format::formatNumberToImengineFloat(1),
            ],
            '3:2' => [
                'x' => Format::formatNumberToImengineFloat(0.07875),
                'y' => Format::formatNumberToImengineFloat(0),
                'width' => Format::formatNumberToImengineFloat(0.84375),
                'height' => Format::formatNumberToImengineFloat(1),
            ],
            '2:3' => [
                'x' => Format::formatNumberToImengineFloat(0.3125),
                'y' => Format::formatNumberToImengineFloat(0),
                'width' => Format::formatNumberToImengineFloat(0.375),
                'height' => Format::formatNumberToImengineFloat(1),
            ],
        ];
        $this->fill([
            'crop' => $cropData
        ]);

        $this->getCropForRatio('16:9')->shouldReturn($cropData['16:9']);
        $this->getCropForRatio('3:2')->shouldReturn($cropData['3:2']);
        $this->getCropForRatio('2:3')->shouldReturn($cropData['2:3']);
    }

    public function it_should_return_empty_crop_if_ratio_cant_be_found(): void
    {
        $cropData = [
            '16:9' => [
                'x' => Format::formatNumberToImengineFloat(0),
                'y' => Format::formatNumberToImengineFloat(0),
                'width' => Format::formatNumberToImengineFloat(1),
                'height' => Format::formatNumberToImengineFloat(1),
            ]
        ];
        $this->fill([
            'crop' => $cropData
        ]);

        $this->getCropForRatio('4:3')->shouldReturn([]);
    }

    public function it_can_determine_if_the_image_has_certain_crop_ration(): void
    {

        $this->hasCropForRatio('16:9')->shouldBe(false);

        $this->set('crop', [
            '16:9' => [
                'x' => Format::formatNumberToImengineFloat(0),
                'y' => Format::formatNumberToImengineFloat(0),
                'width' => Format::formatNumberToImengineFloat(1),
                'height' => Format::formatNumberToImengineFloat(1),
            ]
        ]);

        $this->hasCropForRatio('16:9')->shouldBe(true);
        $this->hasCropForRatio('4:3')->shouldBe(false);
    }

    public function it_can_determine_if_the_image_has_its_original_sizes(): void
    {

        $this->hasOriginalSizes()->shouldBe(false);

        $this->fill([
            'width' => 100,
            'height' => 100,
        ]);

        $this->hasOriginalSizes()->shouldBe(true);
    }

    public function it_should_treat_only_one_original_size_as_not_set(): void
    {
        $this->hasOriginalSizes()->shouldBe(false);

        $this->set('width', 100);

        $this->hasOriginalSizes()->shouldBe(false);

        $this->set('height', 100);

        $this->hasOriginalSizes()->shouldBe(true);
    }

    public function it_should_treat_original_size_zero_as_not_set(): void
    {
        $this->hasOriginalSizes()->shouldBe(false);

        $this->fill([
            'width' => 100,
            'height' => 100,
        ]);

        $this->hasOriginalSizes()->shouldBe(true);

        $this->set('width', 0);

        $this->hasOriginalSizes()->shouldBe(false);
    }

    public function it_should_extract_file_type_from_the_uri(): void
    {
        $this->get('uri')->shouldReturn($this->linkUri);
        $this->fileType()->shouldReturn('jpg');

        $this->set('uri','im://image/HPRP2eo8H05Nqc6EekxdtGo1Al8.png');
        $this->fileType()->shouldReturn('png');
    }

    public function it_should_return_default_file_type_if_uri_cant_be_found(): void
    {
        $this->get('uri')->shouldReturn($this->linkUri);
        $this->fileType()->shouldReturn('jpg');

        $this->set('uri','');
        $this->fileType()->shouldReturn('jpg');
        $this->fileType('png')->shouldReturn('png');
    }
}
